#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from zope.contentprovider.interfaces import IContentProvider

from pyams_form.interfaces.form import IFormSecurityContext
from pyams_viewlet.interfaces import IViewletManager
from pyams_viewlet.manager import viewletmanager_config, WeightOrderedViewletManager, TemplateBasedViewletManager

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.component.extfile.interfaces import IExtFileLinksContainerTarget, IExtFileLinksContainer
from pyams_content.component.gallery.interfaces import IGalleryLinksContainerTarget, IGalleryLinksContainer
from pyams_content.component.links.interfaces import ILinkLinksContainerTarget, ILinkLinksContainer
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, IBaseParagraph
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphTitleToolbar
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IColumn, IValues

# import packages
from pyams_content.shared.common.zmi import WfModifiedContentColumnMixin
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn, TrashColumn, ActionColumn, JsActionColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter, ContextAdapter
from pyams_utils.url import absolute_url
from pyramid.exceptions import NotFound
from pyramid.view import view_config
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.view import AdminView
from pyramid.decorator import reify
from z3c.table.column import GetAttrColumn
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='paragraphs.menu', context=IParagraphContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=VIEW_SYSTEM_PERMISSION, weight=100)
class ParagraphsContainerMenu(MenuItem):
    """Paragraphs container menu"""

    label = _("Paragraphs...")
    icon_class = 'fa-paragraph'
    url = '#paragraphs.html'


#
# Paragraphs container view
#

@pagelet_config(name='paragraphs.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/container.pt', layer=IPyAMSLayer)
@implementer(IInnerPage)
class ParagraphContainerView(AdminView):
    """Paragraphs container view"""

    title = _("Paragraphs list")

    def __init__(self, context, request):
        super(ParagraphContainerView, self).__init__(context, request)
        self.table = ParagraphContainerTable(context, request, self)

    def update(self):
        super(ParagraphContainerView, self).update()
        self.table.update()


class ParagraphContainerTable(ProtectedFormObjectMixin, BaseTable):
    """Paragraphs container table"""

    id = 'paragraphs_list'
    hide_header = True
    sortOn = None

    def __init__(self, context, request, view):
        super(ParagraphContainerTable, self).__init__(context, request)
        self.view = view

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover', 'table-tight']
        permission = self.permission
        if (not permission) or self.request.has_permission(permission, self.context):
            classes.append('table-dnd')
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(ParagraphContainerTable, self).data_attributes
        attributes['table'] = {'id': self.id,
                               'data-ams-plugins': 'pyams_content',
                               'data-ams-plugin-pyams_content-src':
                                   '/--static--/pyams_content/js/pyams_content{MyAMS.devext}.js',
                               'data-ams-location': absolute_url(IParagraphContainer(self.context), self.request),
                               'data-ams-tablednd-drag-handle': 'td.sorter',
                               'data-ams-tablednd-drop-target': 'set-paragraphs-order.json'}
        return attributes

    @reify
    def values(self):
        return list(super(ParagraphContainerTable, self).values)

    def render(self):
        if not self.values:
            translate = self.request.localizer.translate
            return translate(_("No currently defined paragraph."))
        return super(ParagraphContainerTable, self).render()


@adapter_config(name='sorter', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerSorterColumn(ProtectedFormObjectMixin, ActionColumn):
    """Paragraphs container sorter column"""

    cssClasses = {'th': 'action',
                  'td': 'action sorter'}

    icon_class = 'fa fa-fw fa-sort'
    icon_hint = _("Click and drag to sort paragraphs...")

    url = '#'
    weight = 1

    def get_url(self, item):
        return '#'


@adapter_config(name='show-hide', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerShowHideColumn(ProtectedFormObjectMixin, JsActionColumn):
    """Paragraphs container visibility switcher column"""

    cssClasses = {'th': 'action',
                  'td': 'action switcher'}

    icon_class = 'fa fa-fw fa-eye'
    icon_hint = _("Switch paragraph visibility")

    url = 'PyAMS_content.paragraphs.switchVisibility'

    weight = 5

    def get_icon(self, item):
        if item.visible:
            icon_class = 'fa fa-fw fa-eye'
        else:
            icon_class = 'fa fa-fw fa-eye-slash text-danger'
        return '<i class="{icon_class}"></i>'.format(icon_class=icon_class)

    def renderCell(self, item):
        if self.permission and not self.request.has_permission(self.permission, context=item):
            return self.get_icon(item)
        else:
            return super(ParagraphContainerShowHideColumn, self).renderCell(item)


@adapter_config(context=ParagraphContainerShowHideColumn, provides=IFormSecurityContext)
def ShowHideColumnSecurityContextFactory(column):
    """Show/hide column security context factory"""
    return column.table.context


@viewletmanager_config(name='pyams_paragraph.title_toolbar', context=IBaseParagraph, layer=IPyAMSLayer,
                       view=ParagraphContainerTable, provides=IParagraphTitleToolbar)
@template_config(template='templates/paragraph-title-toolbar.pt', layer=IPyAMSLayer)
@implementer(IParagraphTitleToolbar)
class ParagraphTitleToolbarViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Paragraph title toolbar viewlet manager"""


@viewlet_config(name='files', context=IExtFileLinksContainerTarget, layer=IPyAMSLayer, view=ParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=10)
@template_config(template='templates/paragraph-title-icon.pt', layer=IPyAMSLayer)
class ParagraphContainerExtFileLinksAction(Viewlet):
    """Paragraph container external files links column"""

    action_class = 'action extfiles nowrap width-40'
    icon_class = 'fa fa-fw fa-file-text-o'
    icon_hint = _("External files")

    url = 'extfile-links.html'
    modal_target = True

    @property
    def count(self):
        return len(IExtFileLinksContainer(self.context).files or ())


@viewlet_config(name='links', context=ILinkLinksContainerTarget, layer=IPyAMSLayer, view=ParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=20)
@template_config(template='templates/paragraph-title-icon.pt', layer=IPyAMSLayer)
class ParagraphContainerLinkLinksAction(Viewlet):
    """Paragraph container links links column"""

    action_class = 'action links nowrap width-40'
    icon_class = 'fa fa-fw fa-link'
    icon_hint = _("Useful links")

    url = 'link-links.html'
    modal_target = True

    @property
    def count(self):
        return len(ILinkLinksContainer(self.context).links or ())


@viewlet_config(name='gallery', context=IGalleryLinksContainerTarget, layer=IPyAMSLayer, view=ParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=30)
@template_config(template='templates/paragraph-title-icon.pt', layer=IPyAMSLayer)
class ParagraphContainerGalleryLinksAction(Viewlet):
    """Paragraph container gallery links column"""

    action_class = 'action galleries nowrap width-40'
    icon_class = 'fa fa-fw fa-picture-o'
    icon_hint = _("Images galleries")

    url = 'gallery-links.html'
    modal_target = True

    @property
    def count(self):
        return len(IGalleryLinksContainer(self.context).galleries or ())


@adapter_config(name='name', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerTitleColumn(I18nColumn, WfModifiedContentColumnMixin, GetAttrColumn):
    """Paragraph container title column"""

    _header = _("Title")

    weight = 50

    def renderHeadCell(self):
        return '<span class="small hint" title="{title}" data-ams-hint-gravity="e"' \
               '      data-ams-stop-propagation="true"' \
               '      data-ams-click-handler="PyAMS_content.paragraphs.switchAllEditors">' \
               '    <i class="fa fa-plus-square-o"></i>' \
               '</span> '.format(
                    title=self.request.localizer.translate(_("Click to open/close all paragraphs editors"))) + \
               super(ParagraphContainerTitleColumn, self).renderHeadCell()

    def renderCell(self, item):
        registry = self.request.registry
        provider = registry.queryMultiAdapter((item, self.request, self.table), IContentProvider,
                                              name='pyams_paragraph.title_toolbar')
        if provider is not None:
            provider.update()
            provider = provider.render()
        else:
            provider = ''
        return '<div>{provider}<span class="small hint" title="{title}" data-ams-hint-gravity="e"' \
               '      data-ams-stop-propagation="true" ' \
               '      data-ams-click-handler="PyAMS_content.paragraphs.switchEditor">' \
               '    <i class="fa fa-plus-square-o"></i>' \
               '</span> '.format(provider=provider,
                                 title=self.request.localizer.translate(_("Click to open/close paragraph editor"))) + \
               '<span class="title">{0}</span>'.format(super(ParagraphContainerTitleColumn, self).renderCell(item)) + \
               '</div><div class="inner-table-form editor margin-x-10 margin-bottom-0"></div>'

    def getValue(self, obj):
        return II18n(obj).query_attribute('title', request=self.request) or '--'


@adapter_config(name='trash', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Paragraphs container trash column"""


@adapter_config(context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable), provides=IValues)
class ParagraphContainerValues(ContextRequestViewAdapter):
    """Paragraphs container values"""

    @property
    def values(self):
        return IParagraphContainer(self.context).values()


@adapter_config(context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerView), provides=IPageHeader)
class ParagraphHeaderAdapter(DefaultPageHeaderAdapter):
    """Paragraphs container header adapter"""

    back_url = '#properties.html'

    icon_class = 'fa fa-fw fa-paragraph'


@view_config(name='set-paragraphs-order.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_paragraphs_order(request):
    """Update paragraphs order"""
    container = IParagraphContainer(request.context)
    order = list(map(str, json.loads(request.params.get('names'))))
    container.updateOrder(order)
    return {'status': 'success'}


@view_config(name='set-paragraph-visibility.json', context=IParagraphContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_paragraph_visibility(request):
    """Set paragraph visibility"""
    container = IParagraphContainer(request.context)
    paragraph = container.get(str(request.params.get('object_name')))
    if paragraph is None:
        raise NotFound()
    paragraph = IBaseParagraph(paragraph)
    paragraph.visible = not paragraph.visible
    return {'visible': paragraph.visible}


@view_config(name='get-paragraph-editor.json', context=IParagraphContainer, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def get_paragraph_editor(request):
    """Get paragraph editor"""
    container = IParagraphContainer(request.context)
    paragraph = container.get(str(request.params.get('object_name')))
    if paragraph is None:
        raise NotFound()
    registry = request.registry
    editor = registry.queryMultiAdapter((paragraph, request), IParagraphInnerEditor)
    if editor is None:
        editor = registry.queryAdapter(paragraph, IParagraphInnerEditor)
    if editor is not None:
        editor.update()
        return editor.render()


@view_config(name='get-paragraphs-editors.json', context=IParagraphContainer, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def get_paragraphs_editors(request):
    """Get all paragraphs inner editors"""
    container = IParagraphContainer(request.context)
    registry = request.registry
    result = {}
    for key, paragraph in container.items():
        editor = registry.queryMultiAdapter((paragraph, request), IParagraphInnerEditor)
        if editor is None:
            editor = registry.queryAdapter(paragraph, IParagraphInnerEditor)
        if editor is not None:
            editor.update()
            result[key] = editor.render()
    return result
